
/*
 * announce.c
 *
 * From 1997. 5.12
 * by Keiichiro Oguma
 */

#include <stdio.h>
#include <winsock.h>

static void usage(void);


void main(int argc, char **argv)
{
  WORD wVersionRequested;
  WSADATA wsaData;
  int err;
  SOCKET sock;
  SOCKADDR_IN saddr;
  char buf[1024];
  char *msgbuf;
  FILE *fpmsg = NULL;
  u_short port = 23; /* default: telnet */
  int waitsec = 1;
  int silent = FALSE;
  
  if (argc == 1) {
    usage();
    exit(1);
  }
  while(*++argv != NULL) {
    if (**argv != '-' && **argv != '/') {
      usage();
      exit(1);
    }
    switch(*++*argv) {
    case '\0':
      /* message from stdin */
      break;
    case 's':
      silent = TRUE;
      break;
    case 'p':
      /* port */
      if (*++*argv == '\0') {
        if (*++argv == NULL) {
          usage();
          exit(1);
        }
      }
      if ((port = atoi(*argv)) == 0) {
        puts("illegal port number");
        exit(1);
      }
      break;
    case 'w':
      /* wait seconds */
      if (*++*argv == '\0') {
        if (*++argv == NULL) {
          usage();
          exit(1);
        }
      }
      if ((waitsec = atoi(*argv)) == 0) {
        puts("illegal seconds");
        exit(1);
      }
      break;
    case 'f':
      /* file */
      if (fpmsg != NULL) {
        puts("file can only be specified once");
        exit(1);
      }
      if (*++*argv == '\0') {
        if (*++argv == NULL) {
          usage();
          exit(1);
        }
      }
      if ((fpmsg = fopen(*argv, "rt")) == NULL) {
        printf("can't open '%s'\n", *argv);
        exit(1);
      }
      break;
    default:
      usage();
      exit(1);
    }
  }
  wVersionRequested = MAKEWORD(1, 1);
  err = WSAStartup(wVersionRequested, &wsaData);
  if (err != 0 || LOBYTE(wsaData.wVersion) != 1 || HIBYTE(wsaData.wVersion) != 1) {
    puts("Can't intialize WINSOCK or the version is too old");
    goto error;
  }
  if ((sock = socket(PF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET) {
    puts( "Can't make a socket");
    goto error;
  }
  saddr.sin_family = AF_INET;
  saddr.sin_addr.s_addr = 0; 
  saddr.sin_port = htons(port);
  if (bind(sock, (LPSOCKADDR)&saddr, sizeof(saddr)) != 0) {
    printf("Can't bind port: %d", WSAGetLastError());
    goto error;
  }
  if (listen(sock, 5/* fixed */) != 0) {
    printf("Can't listen port: %d", WSAGetLastError());
    goto error;
  }
  if (fpmsg == NULL) {
    if (!silent)
      printf("Enter announce message (type Ctrl+Z to end)\n");
    fpmsg = stdin;
  }
  if ((msgbuf = (char *)malloc(1)) == NULL) {
    puts("out of memory");
    goto error;
  }
  *msgbuf = '\0';
  while(fgets(buf, sizeof(buf) - 1, fpmsg) != NULL) {
    if ((msgbuf = (char *)realloc(msgbuf, strlen(msgbuf) + strlen(buf) + 2)) == NULL) {
      puts("out of memory");
      goto error;
    }
    {
      char *p;
      
      if ((p = strstr(buf, "\r\n")) != NULL
          || (p = strchr(buf, '\r')) != NULL
          || (p = strchr(buf, '\n')) != NULL) {
        strcpy(p, "\r\n");
      }
    }
    strcat(msgbuf, buf);
  }
  fflush(stdin);
  fflush(stdout);
  if (!silent) {
    if (fpmsg == stdin)
      puts("");
    printf("waiting for port %d... (Ctl+C to abort)\n", port);
  }
  for (;;) {
    SOCKET sock_c;
    SOCKADDR_IN saddr_c;
    int slen_c = sizeof(saddr_c);
    
    if ((sock_c = accept(sock, (LPSOCKADDR)&saddr_c, &slen_c)) == INVALID_SOCKET) {
      printf("Can't accept port: %d", WSAGetLastError());
      goto error;
    }
    else {
      int len;
      char *msgp = msgbuf;
      int msglen = strlen(msgbuf);

      do {
        len = send(sock_c, msgp, msglen, 0);
        msgp += len;
        msglen -= len;
      } while(msglen > 0);
      Sleep(waitsec * 1000);
      closesocket(sock_c);
    }
  }
  WSACleanup();
  exit(0);

  error:;
  
  WSACleanup();
  exit(1);
}


static void usage()
{
  fputs(
    "announce: TCP/IP announcer daemon version 1.0\n"
    "\n"
    "usage: announce <-|-s|-f <filename>|-p <portnumber>|-w <seconds>>\n"
    "\n"
    "-             input message from standard input\n"
    "-s            silent mode (no messages)\n"
    "-f FILENAME   input message from file\n"
    "-p PORTNUMBER specify TCP/IP port number (default:23(telnet)\n"
    "-w SECONDS    wait in seconds after message (default:1)\n"
    "\n"
    "Produced 1997 by 'bak' Oguma.\n"
  , stderr);
}

